
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %                                                %
            % Model of Darrieus type Marine Current Turbine  %
            %         Copyright 2006 Tom McCombes            %
            %        University of Strathclyde               %
            %                                                %  
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
%--------------------------------------------------------------------------
%                       Initialise Variables & Constants
%--------------------------------------------------------------------------
                    %-------------CONSTANTS--------

rho=1027;
nu=1.17e-6;
mju=rho*nu;
beta=90;
theta=(1:360);
t_upon_c=18;        %aerofoil thickness
title=horzcat(  '    RPM   ',  '       V0  ',  '    TSR ',  '    chord   ',  '     Cp  ');
                    %-------------INPUTS-----------
Uinf_=input(            'Enter freestream velocities as a vector in m/s [0.5:0.5:7]:............. ');                    
N=input(                'Enter number of blades [3]:............................................. ');
radius=input(           'Enter radius in m:...................................................... ');
C_=input(                'Enter chord divided by radius [0.05]:................................... ');
%theta_mount=input(      'Enter blade twist vector [0]:........................................... ');
maxits=input(           'Enter max number of iterations [100 is best for speed vs. convergence]:. ');
%dr=input(               'Enter dr in metres [0.25 is best for speed vs. accuracy]:............... ');
disp(                   'Enter tolerance for convergence of inflow factors');
tolerance=input(    '                    [0.001 is best for speed vs. accuracy]:............. ');                  

N=3;%input('Enter number of blades: ');
%C=0.3%input('Enter blade chord  in m: ');
%h=3.6%input('Enter blade length  in m: ');
%radius=0.8%input('Enter radius in m: ');
%theta_i=5%input('Enter tangential mounted angle of incidence [0]: ' );
numsegs=1;%input('Enter number of blade elements [100]: ');

clear temp
%Uinf=[0:0.1:5];    %this vector is used if freestream velocities must be pre-specified
%Omega=[0:100:3000];    %ditto but for Omega
Uinf_=1.4%1:.5:7;%input('Enter freestream velocity vector in m/s [0:0.1:5]: '); %single value also accepted
Omega_=150:25:750;%input('Enter angular velocity vector in RPM [0:1:150]: '); %ditto
%isyes=input('Indicate preference for NACA0012 with 1, else 0: '); %choose 1; 0 doesn't exist (yet)
start_ang=2.5;%input('Enter starting angle in degrees [2.5]: '); %2.5 is good: it avoids landing exactly on 0,90,180,270 or 360 degrees
interv_ang=10;%15%input('Enter interval angle in degrees [5]: '); %5 degrees is more expedient in terms of processing times
maxits=100;
tolerance=10^-5
C_=0.375;       %chord length as a fraction of radius
hh=10;%20; %max height
                %-------------VARIBLES---------
                
a=0; a_new=0;
alpha=0;
Cn=0; Ct=0; Cl=0; Cd=0;
T=0; Q=0; M=0; P=0;
Ct_=0; Ur=0; Fx_=0; UuponUinf=0;
Ts=0; Re=0;

  %blade area

%--------------------------------------------------------------------------
%                        Define (Anonymous) Functions
%--------------------------------------------------------------------------

fAlpha          =@  (theta,beta,Ut,U) (((U*(sind(theta))*(sind(beta)))/((U*(cosd(theta)))+Ut))); %
%fCl             =@  (alpha) interp1(ALPHA_FOR_CL,CL,alpha,'linear','extrap');
%fCd             =@  (alpha) interp1(ALPHA_FOR_CD,CD,alpha,'linear','extrap');
fCt             =@  (alpha,Cl,Cd) ((Cl*sind(alpha))-(Cd*cosd(alpha)));
%fCt_            =@  (ALPHA,Ct_,alpha,CD0) (interp1(ALPHA,Ct_,alpha,'linera','extrap'))-CD0;
fCn             =@  (alpha,Cl,Cd) ((Cl*cosd(alpha))+(Cd*sind(alpha)));
%fCn_            =@  (ALPHA,Cn,alpha) interp1(ALPHA,Cn,alpha,'linera','extrap');
fUr             =@  (alpha,U,theta,beta) (U*(sind(theta))*sind(beta))/(sind(alpha)); %relative velocity
fFx_            =@  (N,C,r,Ur,Uinf,Cn,Ct,theta,beta) ((N*C/(4*pi*r))*((Ur/Uinf)^2)*(Cn-Ct*((cosd(theta))/((sind(theta))*(sind(beta))))));
fa_new          =@  (Fx_,a) (Fx_+a^2);
fU              =@  (a,Uinf) -(a-1)/Uinf;
fUuponUinf      =@  (a,Uinf) -(a-1);
fTs             =@  (rho,r,Ct,C,deltah,beta,Ur) (0.5*rho*r*Ct*C*deltah*Ur*Ur/(sind(beta)));  %torque on individual blade element as it passes thru streamtube
fRe             =@  (rho, mju, C, Ur) rho*C*Ur/mju;

%--------------------------------------------------------------------------
%                        Initiate Algorhithm
%--------------------------------------------------------------------------

for r_=1%5:2:20%:10
    radius=0.8%r_;
    C=C_*radius;
    tic;
       a=0; a_new=0;
alpha=0;
Cn=0; Ct=0; Cl=0; Cd=0;
T=0; Q=0; M=0; P=0;
Ct_=0; Ur=0; Fx_=0; UuponUinf=0;
Ts=0; Re=0;
    Omega_=100:10:400%10:10:round(((60/(2*pi))*round((12/radius)+1))/10)*10;
    for h_=1%5:2:20%1:hh%20
        height=1.6%r_;%h_;
        h=1.6%height;
        S=C*h;
        deltah=h/numsegs;
        for ome=1:max(size(Omega_))
            omega=Omega_(ome)*2*pi/60;
            Ut=omega*radius;

            hold_Ut(ome)=Ut;

            for vel=1:max(size(Uinf_))
                Uinf=Uinf_(vel);
                %disp(max(size(Uinf_)/vel));
                Count=1;
                %solve momentum
                for Theta=start_ang:interv_ang:360%+start_ang
                    %disp(Theta)
                    Num_angs=max(size(start_ang:interv_ang:360+start_ang));

                    if Theta==180
                        theta=180.1;
                    elseif Theta==360
                        theta=360.1;
                    elseif Theta==0
                        theta=0.1;
                    elseif Theta==270
                        theta=270.1;
                    elseif Theta==90
                        theta=90.1;
                    else 
                        theta=Theta;
                    end
                    
                    
                    stheta=sind(Theta);
                    sbeta=sind(beta);
                    ctheta=cosd(Theta);
                    
                    
                    if stheta==0
                        stheta=10^-10;
                    else
                        stheta=stheta;
                    end
                    
                    upwind=0;
            
                    if sind(Theta)>=0
                    upwind=1;
                    end
                    
                    ytheta(Count)=radius*cosd(Theta);
                    
                    for n=1:numsegs
                        %disp(n)
                        isfinished=0;
                        %initial conditions
                        a=0;
                
                        its=0;
                
%--------------------------------------------------------------------------
%                               Algorithm
%--------------------------------------------------------------------------
                        if upwind==1
                            Uinf__=Uinf;
                            ytheta_uw(Count)=radius*cosd(Theta);
                            U=Uinf__;
                    
                        else
                            ytheta_dw(Count)=radius*cosd(Theta);
                            a_uw=interp1(ytheta_uw,hold_a_uw,ytheta_dw,'linear','extrap');
                            Uinf__=Uinf*(1-2*a_uw(Count));
                            U=Uinf__;
                        end
                        
                        alpha=atand((Uinf*stheta)/((Uinf*ctheta)+Ut));
                
                            while (isfinished==0)   && its<maxits   %need to put in a thing to interpolate between reynolds numbers 
                                %so as to get the correct values of Cl etc and change the values for following calcs accordingly
                                % not too important at moment
                                its=its+1;
                                %disp(its); %this will show an iteration counter on screen

                                %disp(Theta)

                                %if isnan(alpha)==1 || isinf(alpha)==1
                                %    alpha=90;
                                %end
                                salpha=sind(alpha);
                                calpha=cosd(alpha);

                                hold_alpha(Count,n)=alpha;

                                %disp(alpha) %this will display alpha on screen
                                if alpha>0
                                    aero_coeffs=NACA_RE_5million(t_upon_c,alpha);
                                    Cl=aero_coeffs(1);
                                    Cd=aero_coeffs(2);
                                else 
                                    aero_coeffs=NACA_RE_5million(t_upon_c,-alpha);
                                    Cl=-aero_coeffs(1);
                                    Cd=aero_coeffs(2);
                                end




                                Cn=(Cl*calpha)+(Cd*salpha);
                                Ct=(Cl*salpha)-(Cd*calpha);





                                Ur= (U*stheta*sbeta/salpha);

                                %Ur=fUr(alpha,U,theta,beta);
                                temp1=N*C/(4*180*radius*deltah);
                                temp2=(Ur/Uinf)^2;
                                temp3=Cn-(Ct*ctheta/(stheta*sbeta));
                                Fx_=temp1*temp2*temp3;
                                clear temp*;
                                %Fx_=fFx_(N,C,radius,Ur,Uinf_,Cn,Ct,theta,beta);
                                anew=(a^2)+Fx_;


                                if abs(a-anew)<tolerance || its>maxits
                                        isfinished=1;
                                end

                                a=anew;%(((atemp+a)/2) + a)/2;%atemp;
                                clear atemp anew

                                %UuponUinf =-(a-1);%fUuponUinf(a_new,Uinf_);
                                %disp(UuponUinf);
                                U=Uinf*(1-a);%UuponUinf*Uinf_;
                                alpha=atand((U*stheta)/((U*ctheta)+Ut));%fAlpha(theta,beta,Ut,U);
                                msg=[num2str(alpha),'   ',num2str(U),'  ',num2str(Ur)];
                                %disp(msg);
                            end
                            
                            
                            %--------------------display calculation progress information
%if max(size(theta_mount))==1
%hold_1=horzcat('Calculating...');
%hold_2=horzcat('Velocity number         ',num2str(vel),  '  of ',num2str(max(size(Uinf_))), '   current velocity is: ',num2str(Uinf_(vel)),' m/s');
%hold_3=horzcat('RPM number              ',num2str(ome),  '  of ',num2str(max(size(Omega_))),    '   current angular velocity is: ',num2str(Omega_(ome)),' RPM');
%hold_6=horzcat('Current height     ',num2str(height),    '   of  ',num2str(hh),  '   current height is: ',num2str(height),' m');
%hold_5=horzcat('Radius: ',num2str(radius),' m. Iterations: ',num2str(its),    '  of ',num2str(maxits),' iterations and is to a tolerance of ',num2str(tolerance));
%hold_4=horzcat('Blade calc is           ',num2str(100*radial/(max(size(rstep)))),    '   % complete');

%disp(hold_1);
%disp(hold_6);
%disp(hold_2);
%disp(hold_3);
%disp(hold_4);
%disp(hold_5);
%end
             
                    
            

    


%-----------------------END OF ALGORITHM----------------------
                        if upwind==1
                            hold_a_uw(n,Count)=a;
                        end
                        ahold(n,Count)=a; %this creates an array holding all computed a's
                        Ts(n)=0.5*rho*radius*Ct*C*deltah*Ur*Ur;%fTs(rho,radius,Ct,C,deltah,beta,Ur);
                        %if isnan(Ts(numsegs))==1
                        %    Ts(numsegs)=0
                        %end
                        Count=Count+1;
                        %disp(Count);
                        end
               
                        %if isnan(sum(Ts))==0 &&  isinf(sum(Ts))==0

                            Tb(Count)=sum(Ts);

                        %else

                            %Tb(Count)=0;

                        %end

                    end
                    Tave(vel,ome,h_)=N*mean(Tb)/(180/interv_ang);
                    Pavail(vel,ome,h_)=rho*Uinf*Uinf*Uinf*h*radius;
                    P(vel,ome,h_)=Tave(vel,ome,h_)*omega;
                    CP(vel,ome,h_)=P(vel,ome,h_)/Pavail(vel,ome,h_);
                    TSR(vel,ome,h_)=Ut/Uinf;
                    
                    if CP(vel,ome,h_)<0
                        break
                    end
            end
        end
    end
    
        
        
        
        
        
    disp(' ');
    disp(' ');
    disp(' ');
    disp('Double-Multiple Streamtube Model for a Darrieus type MCT');
    msg1=horzcat('Current Radius: ',num2str(radius),' m');
    disp(msg1);
    clear msg1;
    toc;
    disp('.........................................................');
title=horzcat(  '  Velocity [m/s] ',  '   TSR ',  '     height [m] ',  '       Cpmax ',  '     Pmax [kW] ');
disp(title);
for l=1:max(size(Uinf_))
    vel=Uinf_(l);
for m=1:max(size(Omega_))
for n=1:hh  %height
CPvel(m,n)=CP(l,m,n);
TSRvel(m,n)=TSR(l,m,n);
Pvel(m,n)=P(l,m,n);
end
end
[a,b]=find(CPvel==max(max(CPvel)),1);
ome_CPmax=Omega_(a);
height_CPmax=b;
CPmax=max(max(CPvel));
TSR_CPmax=TSRvel(a,b);
Pmax=Pvel(a,b);
msg_line_str=[  '       ',num2str(vel), '          ',num2str(TSR_CPmax), '         ',num2str(height_CPmax),...
    '             ',num2str(CPmax),   '     ',num2str(Pmax/1000)];
msg_line=horzcat(vel,TSR_CPmax,height_CPmax,CPmax,Pmax);
if l==1
    RESULTS=msg_line;
else
    RESULTS=vertcat(RESULTS,msg_line);
end
disp(msg_line_str);
end


%----------------------------saving
temp1=horzcat(num2str(radius),'_',num2str(tolerance),'_',num2str(max(size(Omega_))),'_',num2str(max(size(Uinf_))),'.mat');
name=horzcat('Results\',temp1);
save (name,'CP','TSR','Pavail','P','RESULTS','interv_ang');



msg=horzcat('Output file saved as: ',temp1,' in Results folder');
disp('.........................................................');
disp(msg);
disp('.........................................................');
clear A B CP	CPmax	CPvel	Cd	Cl	Cn	Ct	Cthrust	F	FN	FT	Fh	Ft	LAMBDA	M...
Mtot	OM P 	Pavail Pmax	Pvel	Q	RESULTS	T	THETA	TSR	TSR_CPmax	TSRvel	Vrel	Vrel2	a	ac	adash	adashnew...	
adashold	aero_coeffs	alpha	anew	aold	b	cphi	dM	dT	error_flag	error_holder	f1	...
f2	root  offset tip...
fAi	...
fBi	...
fC	...
fCn	...
fCt	...
fF	...
fK	...
fVrel	...
fa1	...
fa2	...
fadash...	
fainit	...
fdM	...
fdP...
fdT	...
ffh	...
fft	...
fphi	...
fsigma	...
ftheta	...
ftheta_twist ...	
fthickness	...
hold	...
hold_1	...
hold_2	...
hold_3	...
hold_4	...
hold_5	...
hold_6	...
hold_CP	...
hold_Cd	...
hold_Cl	...
hold_Cn	...
hold_Ct	...
hold_Cthrust ...	
hold_F	...
hold_Vr	...
hold_Vrel	...
hold_adash...	
hold_alpha	...
hold_m	...
hold_phi	...
hold_sigma	...
hold_vi	....
i	...
isfinished	...
l	...
m	...
msg	...
msg_line ...	
msg_line_str ...	
n	...
name	...
nom1	...
number_ome...	
number_vel	...
ome	...
ome_CPmax ...	
omega	...
omemax	...
oneminusa	...
oneminusa2	...
oneplusadash	...
p	...
p0	...
phi	...
pvap	...
r	...
r_upon_R ...	
rad	...
radial... 	
radval	...
rdiff2	...
rdiff3	...
sphi	...
sphi2	...
taper	...
temp1	...
theta...
theta_CPmax	...
theta_change	...
theta_twist	...
tip_speed	...
title	...
vel
pack;

end
        
