%function [Pout,Avail] = darrieus_func(n,L,h,radius,thetai)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                %
% Model of Darrieus type Marine Current Turbine  %
%         Copyright 2006 Tom McCombes            %
%        University of Strathclyde               %
%                                                %  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
%--------------------------------------------------------------------------
%                       Initialise Variables & Constants
%--------------------------------------------------------------------------
                    %-------------CONSTANTS--------

rho=1027;
nu=1.17e-6;
mju=rho*nu;
beta=90;
theta=(1:360);
t_upon_c=12;        %aerofoil thickness
                    %-------------INPUTS-----------
                    
N=3%input('Enter number of blades: ');
C=0.1152;%0.3%input('Enter blade chord  in m: ');
h=0.2%1.8%input('Enter blade length  in m: ');
radius=0.15%0.8%input('Enter radius in m: ');
%theta_i=5%input('Enter tangential mounted angle of incidence [0]: ' );
numsegs=1%input('Enter number of blade elements [100]: ');
deltah=h/numsegs
clear temp
%Uinf=[0:0.1:5];    %this vector is used if freestream velocities must be pre-specified
%Omega=[0:100:3000];    %ditto but for Omega
Uinf_=1.4%input('Enter freestream velocity vector in m/s [0:0.1:5]: '); %single value also accepted
Omega_=70:10:200%input('Enter angular velocity vector in RPM [0:1:150]: '); %ditto
%isyes=input('Indicate preference for NACA0012 with 1, else 0: '); %choose 1; 0 doesn't exist (yet)
start_ang=7.5%input('Enter starting angle in degrees [2.5]: '); %2.5 is good: it avoids landing exactly on 0,90,180,270 or 360 degrees
interv_ang=15%input('Enter interval angle in degrees [5]: '); %5 degrees is more expedient in terms of processing times

                %-------------VARIBLES---------
                
a=0; a_new=0;
alpha=0;
Cn=0; Ct=0; Cl=0; Cd=0;
T=0; Q=0; M=0; P=0;
Ct_=0; Ur=0; Fx_=0; UuponUinf=0;
Ts=0; Re=0;

S=C*h;  %blade area

%--------------------------------------------------------------------------
%                        Define (Anonymous) Functions
%--------------------------------------------------------------------------

fAlpha          =@  (theta,beta,Ut,U) (((U*(sind(theta))*(sind(beta)))/((U*(cosd(theta)))+Ut))); %
%fCl             =@  (alpha) interp1(ALPHA_FOR_CL,CL,alpha,'linear','extrap');
%fCd             =@  (alpha) interp1(ALPHA_FOR_CD,CD,alpha,'linear','extrap');
fCt             =@  (alpha,Cl,Cd) ((Cl*sind(alpha))-(Cd*cosd(alpha)));
%fCt_            =@  (ALPHA,Ct_,alpha,CD0) (interp1(ALPHA,Ct_,alpha,'linera','extrap'))-CD0;
fCn             =@  (alpha,Cl,Cd) ((Cl*cosd(alpha))+(Cd*sind(alpha)));
%fCn_            =@  (ALPHA,Cn,alpha) interp1(ALPHA,Cn,alpha,'linera','extrap');
fUr             =@  (alpha,U,theta,beta) (U*(sind(theta))*sind(beta))/(sind(alpha)); %relative velocity
fFx_            =@  (N,C,r,Ur,Uinf,Cn,Ct,theta,beta) ((N*C/(4*pi*r))*((Ur/Uinf)^2)*(Cn-Ct*((cosd(theta))/((sind(theta))*(sind(beta))))));
fa_new          =@  (Fx_,a) (Fx_+a^2);
fU              =@  (a,Uinf) -(a-1)/Uinf;
fUuponUinf      =@  (a,Uinf) -(a-1);
fTs             =@  (rho,r,Ct,C,deltah,beta,Ur) (0.5*rho*r*Ct*C*deltah*Ur*Ur/(sind(beta)));  %torque on individual blade element as it passes thru streamtube
fRe             =@  (rho, mju, C, Ur) rho*C*Ur/mju;

%--------------------------------------------------------------------------
%                        Initiate Algorhithm
%--------------------------------------------------------------------------


   
for ome=1:max(size(Omega_))
    omega=Omega_(ome)*2*pi/60
    Ut=omega*radius;

hold_Ut(ome)=Ut;

    for vel=1:max(size(Uinf_))
        Uinf=Uinf_(vel);
        disp(max(size(Uinf_)/vel));
        Count=1;
        %solve momentum
        for Theta=start_ang:interv_ang:360+start_ang
            %disp(theta)
            Num_angs=max(size(start_ang:interv_ang:360+start_ang));

            if Theta==180
                theta=180.1;
            elseif Theta==360
                theta=360.1;
            elseif Theta==0
                theta=0.1;
            elseif Theta==270
                theta=270.1;
            elseif Theta==90
                theta=90.1;
            else 
               theta=Theta;
            end
            stheta=sind(Theta);
                    sbeta=sind(beta);
                    ctheta=cosd(Theta);
                    if stheta==0
                        stheta=0.0000000000000000000001;
                    else
                        stheta=stheta;
                    end

                    for n=1:numsegs
            %disp(n)
                isfinished=0;
                %initial conditions
                a=0;
                U=Uinf;
                its=0;
                alpha=atand((Uinf*stheta)/((Uinf*ctheta)+Ut));
%--------------------------------------------------------------------------
%                               Algorithm
%--------------------------------------------------------------------------
       
                while (isfinished==0)   && its<10000   %need to put in a thing to interpolate between reynolds numbers 
                    %so as to get the correct values of Cl etc and change the values for following calcs accordingly
                    % not too important at moment
                    its=its+1;
                    %disp(its); %this will show an iteration counter on screen
                    
                    disp(Theta)
                    
                    %if isnan(alpha)==1 || isinf(alpha)==1
                    %    alpha=90;
                    %end
                    salpha=sind(alpha);
                    calpha=cosd(alpha);
                    
                    hold_alpha(Count,n)=alpha;

                    %disp(alpha) %this will display alpha on screen
                    if alpha>0
                            aero_coeffs=NACA_RE_5million(t_upon_c,alpha);
                            Cl=aero_coeffs(1);
                            Cd=aero_coeffs(2);
                    else 
                            aero_coeffs=NACA_RE_5million(t_upon_c,-alpha);
                            Cl=-aero_coeffs(1);
                            Cd=aero_coeffs(2);
                    end
                    
                    
                    
                    
                    Cn=(Cl*calpha)+(Cd*salpha);
                    Ct=(Cl*salpha)-(Cd*calpha);
                    %Fx=



                    
                    Ur= (U*stheta*sbeta/salpha);

                    %Ur=fUr(alpha,U,theta,beta);
                    temp1=N*C/(4*180*radius*deltah);
                    temp2=(Ur/Uinf)^2;
                    temp3=Cn-(Ct*ctheta/(stheta*sbeta));
                    Fx_=temp1*temp2*temp3;
                    clear temp*;
                    %Fx_=fFx_(N,C,radius,Ur,Uinf,Cn,Ct,theta,beta);
                    anew=(a^2)+Fx_;
                    

                    if abs(a-anew)<1*10^-5 || its>5000
                        isfinished=1;
                    end

                    a=anew;%(((atemp+a)/2) + a)/2;%atemp;
                    clear atemp anew
                    
                    %UuponUinf =-(a-1);%fUuponUinf(a_new,Uinf);
                    %disp(UuponUinf);
                    U=Uinf*(1-a);%UuponUinf*Uinf;
                    alpha=atand((U*stheta)/((U*ctheta)+Ut));%fAlpha(theta,beta,Ut,U);
                    msg=[num2str(alpha),'   ',num2str(U),'  ',num2str(Ur)];
					%disp(msg);
                end
                
                    

%-----------------------END OF ALGORITHM----------------------
            ahold(n,Count)=a; %this creates an array holding all computed a's
            Ts(n)=0.5*rho*radius*Ct*C*deltah*Ur*Ur;%fTs(rho,radius,Ct,C,deltah,beta,Ur);
            %if isnan(Ts(numsegs))==1
            %    Ts(numsegs)=0
            %end
        
            end
        Count=Count+1       
        %if isnan(sum(Ts))==0 &&  isinf(sum(Ts))==0
            
            Tb(Count)=sum(Ts);
        
        %else
            
            %Tb(Count)=0;
        
        %end

        end
        Tave(ome,vel)=N*mean(Tb)/(180/Num_angs);
        Pavail(ome,vel)=rho*Uinf*Uinf*Uinf*h*radius;
        P(ome,vel)=Tave(ome,vel)*omega;
        Cp(ome,vel)=P(ome,vel)/Pavail(ome,vel);
        lambda(ome,vel)=Ut/Uinf;
        
    end
end

